﻿using Android.App;
using Android.Widget;
using Android.OS;
//Add following for the sample
using SQLDatabase.Net.PCL.SQLDatabaseClient;
using System.Collections.Generic;
using System.IO;


namespace SQLDatabase.Net.PCL.AndroidExample
{
    [Activity(Label = "SQLDatabase.Net.PCL.AndroidExample", MainLauncher = true, Icon = "@drawable/icon")]
    public class MainActivity : Activity
    {

        TextView lblOutput;
        string StoragePath = string.Empty;

        ListView lstListView;
        List<string> lstItems = new List<string>();
        ArrayAdapter<string> adapter;

        AlertDialog.Builder alert;
        

        protected override void OnCreate(Bundle bundle)
        {
            base.OnCreate(bundle);

            using (SqlDatabaseConnection cn = new SqlDatabaseConnection())
            {
                //can be removed just for example if dll is working.
            }

            // Set our view from the "main" layout resource
            // SetContentView (Resource.Layout.Main);

            base.OnCreate(bundle);

            // Set our view from the "main" layout resource
            // SetContentView (Resource.Layout.Main);

            // Setup External Storage Path from Android.OS.Environment.ExternalStorageDirectory.AbsolutePath;
            StoragePath = Environment.ExternalStorageDirectory.AbsolutePath;


            var layout = new LinearLayout(this);
            layout.Orientation = Orientation.Vertical;

            lblOutput = new TextView(this);
            lblOutput.Text = "SQLDatabase.Net";

            //Set up @memory Example button
            Button AtMemory = new Button(this);
            AtMemory.Text = "@Memory Example";
            AtMemory.Click += AtMemory_Click;

            //Set up StreamObject sample button
            Button StreamObject = new Button(this);
            StreamObject.Text = "Database File";
            StreamObject.Click += StreamObject_Click;

            //Set up DictionaryStream sample button
            Button DictionaryStream = new Button(this);
            DictionaryStream.Text = "Multiple Database File";
            DictionaryStream.Click += DictionaryStream_Click;

            //Set up DictionaryMemoryStream sample button
            Button DictionaryMemoryStream = new Button(this);
            DictionaryMemoryStream.Text = "Memory Stream";
            DictionaryMemoryStream.Click += DictionaryMemoryStream_Click;

            //Set up Read only and auto close example button
            Button ReadOnlyAutoClose = new Button(this);
            ReadOnlyAutoClose.Text = "ReadOnly AutoClose";
            ReadOnlyAutoClose.Click += ReadOnlyAutoClose_Click; ;

            //Set up DictionaryMemoryStream sample button
            Button DictionaryDirect = new Button(this);
            DictionaryDirect.Text = "Direct Dictionary Usage";
            DictionaryDirect.Click += DictionaryDirect_Click; ;

            
            //set alert for executing the task
            alert = new AlertDialog.Builder(this);
            alert.SetTitle("Commit or RollBack ?");
            alert.SetMessage("Create commit transaction example ?");

            alert.SetPositiveButton("Yes", (senderAlert, args) => {
                Toast.MakeText(this, "Commit Example", ToastLength.Short).Show();

                TransactionExample(true);
            });


            alert.SetNegativeButton("No", (senderAlert, args) => {
                Toast.MakeText(this, "Rollback Example", ToastLength.Short).Show();

                TransactionExample(false);
            });

            //Set up Transaction example
            Button Transaction = new Button(this);
            Transaction.Text = "Transaction";
            Transaction.Click += (sender, e) =>
            {              
                Dialog dialog = alert.Create();
                dialog.Show();
            };

            layout.AddView(lblOutput);
            layout.AddView(AtMemory);
            layout.AddView(StreamObject);
            layout.AddView(DictionaryStream);
            layout.AddView(DictionaryMemoryStream);
            layout.AddView(ReadOnlyAutoClose);
            layout.AddView(DictionaryDirect);
            layout.AddView(Transaction);


            lstListView = new ListView(this);
            adapter = new ArrayAdapter<string>(this, Android.Resource.Layout.SimpleListItem1, lstItems);
            lstListView.Adapter = adapter;
            layout.AddView(lstListView);

            SetContentView(layout);


        }

        private void TransactionExample(bool CommitTransaction)
        {
           
            string OutPutMessage = string.Empty;
            using (SqlDatabaseConnection cnn = new SqlDatabaseConnection("SchemaName=db;Datasource=@memory;")) {

                try {
                    cnn.Open();                
                    if (cnn.State == ConnectionState.Open) {                    
                        using (SqlDatabaseCommand cmd = new SqlDatabaseCommand())
                        {
                            cmd.Connection = cnn;                           
                            cmd.CommandText = "CREATE TABLE IF NOT EXISTS db.TEST_TABLE ( ColA INTEGER , ColB TEXT ) ;";
                            cmd.ExecuteNonQuery();
                            cmd.CommandText = "BEGIN;"; // BEGIN TRANSACTION can also be used
                            cmd.ExecuteNonQuery();                            
                            for (int i = 1; i <= 10; i++)
                            {
                                cmd.CommandText = "INSERT INTO db.TEST_TABLE VALUES(" + i + ", 'ColB Value " + i + "') ;";                                
                                cmd.ExecuteNonQuery();
                            }
                            if (CommitTransaction)
                            {
                                cmd.CommandText = "COMMIT;"; //END TRANSACTION can also be used.
                                cmd.ExecuteNonQuery();
                                OutPutMessage = "Transaction Committed";
                            } else
                            {
                                cmd.CommandText = "ROLLBACK;"; 
                                cmd.ExecuteNonQuery();
                                OutPutMessage = "Transaction Rolled back.";
                            }

                            cmd.CommandText = "SELECT COUNT(*) FROM db.TEST_TABLE;";
                            OutPutMessage += " Record Count : " + cmd.ExecuteScalar().ToString();
                        }                    
                }
                }
                catch (System.Exception ex)
                {
                    System.Diagnostics.Debug.WriteLine(ex.Message);
                    OutPutMessage = ex.Message;
                }
                finally
                {
                    if (cnn.State == ConnectionState.Open)
                        cnn.Close();
                }

            }
            lblOutput.Text = OutPutMessage;
        }


        private void ReadOnlyAutoClose_Click(object sender, System.EventArgs e)
        {

            string filename = Path.Combine(StoragePath, "ReadOnlyAutoClose.db");

            using (SqlDatabaseConnection cnn = new SqlDatabaseConnection())
            {
                cnn.Stream = new FileStream(filename, FileMode.OpenOrCreate);
                cnn.CloseStreamsOnDispose = true; //Stream will be closed when SqlDatabaseConnection is disposed.

                cnn.ConnectionString = "SchemaName=db;Datasource=file://" + filename + "; ";

                //UnComment After running once
                //Comment out to test by creating tables and then reading
                
                //cnn.DatabaseMode = DatabaseMode.ReadOnly; //database is in read only mode default is ReadWrite
                
                try
                {
                    cnn.Open();
                }
                catch (System.Exception ex)
                {
                    System.Diagnostics.Debug.WriteLine(ex.Message);
                }

                if (cnn.State == ConnectionState.Open)
                {
                    try
                    {
                        using (SqlDatabaseCommand cmd = new SqlDatabaseCommand())
                        {
                            cmd.Connection = cnn;

                            //Following will fail since READ ONLY database cannot drop, create, insert or update
                            cmd.CommandText = "DROP TABLE IF EXISTS TEST_TABLE;";
                            cmd.ExecuteNonQuery();

                            //read only can not create table
                            cmd.CommandText = "CREATE TABLE IF NOT EXISTS TEST_TABLE ( ColA INTEGER , ColB TEXT ) ;";
                            cmd.ExecuteNonQuery();

                            cmd.CommandText = "INSERT INTO db.TEST_TABLE VALUES(1, 'Record 1') ;";
                            cmd.ExecuteNonQuery();

                            cmd.CommandText = "INSERT INTO db.TEST_TABLE VALUES(2, @data) ;";
                            cmd.Parameters.Add(new SqlDatabaseParameter { ParameterName = "@data", DbType = DbType.Text, Value = "Record 2" });
                            cmd.ExecuteNonQuery();

                            // If Set to readonly update will not work and no error will be raised...
                            cmd.CommandText = "UPDATE db.TEST_TABLE SET ColB = 'Updated Record Attempt 1' WHERE ColA = 1 ;";
                            cmd.ExecuteNonQuery();

                            cmd.CommandText = "SELECT ColB FROM TEST_TABLE WHERE ColA = 1 ;";
                            SqlDatabaseDataReader dr = cmd.ExecuteReader();
                            
                            while (dr.Read())
                            {
                                adapter.Add(dr["ColB"].ToString());
                            }
                            dr.Close();

                            cmd.Dispose(); //Optional
                        }

                    }
                    catch (System.Exception ex)
                    {
                        // should produce message that such and such table does not exists if READONLY
                        adapter.Add(ex.Message);
                    }
                    finally
                    {
                        if (cnn.State == ConnectionState.Open)
                            cnn.Close();
                        
                    }
                }

            }
           
        }

        private void DictionaryDirect_Click(object sender, System.EventArgs e)
        {

            // MemoryStream Usage instead of creating seperate object directly use the property as object

            // you can always add in the stream dictionary and that file will be used.
            //if you are trying to implement your own encryption or saving file to some other manner.

            using (SqlDatabaseConnection cnn = new SqlDatabaseConnection())
            {
                cnn.DictionaryStringMemoryStream = new Dictionary<string, MemoryStream>();

                cnn.ConnectionString = "SchemaName=db;DataSource=file://FileOne;";

                try
                {
                    cnn.Open();
                    using (SqlDatabaseCommand cmd = new SqlDatabaseCommand(cnn))
                    {
                        cmd.CommandText = "ATTACH DATABASE 'FileTwo' AS 'memdb2' ;";
                        cmd.ExecuteNonQuery();

                        cmd.CommandText = "ATTACH DATABASE '@memory' AS 'memdb3' ;";
                        cmd.ExecuteNonQuery();



                        cmd.CommandText = "SYSCMD database_list;";
                        SqlDatabaseDataReader rd = cmd.ExecuteReader();
                        while (rd.Read())
                        {
                            string Line = string.Empty;
                            for (int i = 0; i < rd.VisibleFieldCount; i++)
                                Line += rd.GetValue(i) + " - ";

                            //lblOutput.Text += rd[1] + ",";                            
                            adapter.Add(Line);
                        }

                        if (!rd.IsClosed)
                            rd.Close();
                    }
                }
                catch (System.Exception ex)
                {
                    System.Diagnostics.Debug.WriteLine(ex.Message);
                }
                finally
                {
                    if (cnn.State == ConnectionState.Open)
                        cnn.Close();


                    // Use Lock object to be on safe side
                    lock (cnn.DictionaryMemoryStreamLockObject)
                    {
                        if (cnn.DictionaryStringMemoryStream.Count > 0) //there are stream files
                        {
                            foreach (KeyValuePair<string, MemoryStream> kv in cnn.DictionaryStringMemoryStream)
                            {
                                //Save the memory stream to file or ftp, network location etc..
                                kv.Value.Close();
                            }

                        }

                        // Application MUST clear the stream unless you want to reopen with same stream.
                        // For Auto Close set CloseStreamsOnDispose = true
                        cnn.DictionaryStringMemoryStream.Clear();
                    }
                }
            }




            // File Usage instead of creating seperate object directly use the property as object

            string fileName1 = Path.Combine(StoragePath, "dataFile1.db"); //Primary database         
            string fileName2 = Path.Combine(StoragePath, "dataFile2.db");

            using (SqlDatabaseConnection cnn = new SqlDatabaseConnection())
            {                
                cnn.DictionaryStringStream.Add(fileName1, new FileStream(fileName1, FileMode.OpenOrCreate));
                cnn.DictionaryStringStream.Add(fileName2, new FileStream(fileName2, FileMode.OpenOrCreate));
                cnn.DictionaryStringStream = cnn.DictionaryStringStream; //Assign it to itself.

                cnn.ConnectionString = "SchemaName=db;DataSource=file://" + fileName1 + ";";

                try
                {
                    cnn.Open();
                    using (SqlDatabaseCommand cmd = new SqlDatabaseCommand(cnn))
                    {
                        //code goes here
                    }
                }
                catch (System.Exception ex)
                {
                    System.Diagnostics.Debug.WriteLine(ex.Message);
                }
                finally
                {
                    if (cnn.State == ConnectionState.Open)
                        cnn.Close();


                    // Use Lock object to be on safe side
                    lock (cnn.DictionaryStringStreamLockObject)
                    {
                        if (cnn.DictionaryStringStream.Count > 0) //there are stream files
                        {
                            foreach (KeyValuePair<string, Stream> kv in cnn.DictionaryStringStream)
                                kv.Value.Close();
                        }
                        // Application MUST clear the stream unless you want to reopen with same stream.
                        // Library does not clear dictionary on it's own, to allow application to manage stream.
                        // For Auto Close set CloseStreamsOnDispose = true
                        cnn.DictionaryStringStream.Clear();
                    }
                }
            }

            try
            {
                if (File.Exists(fileName1))
                    File.Delete(fileName1);

                if (File.Exists(fileName2))
                    File.Delete(fileName2);

            }
            catch { }


            

        }

        private void DictionaryMemoryStream_Click(object sender, System.EventArgs e)
        {
            Dictionary<string, MemoryStream> dictionaryMemoryStream = new Dictionary<string, MemoryStream>();


            using (SqlDatabaseConnection cnn = new SqlDatabaseConnection())
            {

                cnn.DictionaryStringMemoryStream = dictionaryMemoryStream;

                cnn.ConnectionString = "SchemaName=db;Datasource=file://FileOne;"; //Provide primary file name

                try
                {
                    cnn.Open();
                }
                catch (System.Exception ex)
                {
                    System.Diagnostics.Debug.WriteLine(ex.Message);
                }

                if (cnn.State == ConnectionState.Open)
                {
                    try
                    {
                        using (SqlDatabaseCommand cmd = new SqlDatabaseCommand())
                        {
                            cmd.Connection = cnn;

                            //cmd.CommandText = "DROP TABLE IF EXISTS TEST_TABLE;";
                            //cmd.ExecuteNonQuery();

                            cmd.CommandText = "CREATE TABLE IF NOT EXISTS  db.TEST_TABLE (AutoIncId INTEGER PRIMARY KEY ASC AUTOINCREMENT, ColA INTEGER , ColB TEXT ) ;";
                            cmd.ExecuteNonQuery();

                            //Attach the second file
                            cmd.CommandText = "ATTACH DATABASE 'FileTwo' AS 'db2' ; "; //db2 is the SchemaName and first is the file name to attach.
                            cmd.ExecuteNonQuery();

                            // use SchemaName . Object
                            cmd.CommandText = "CREATE TABLE IF NOT EXISTS db2.TEST_TABLE (AutoIncId INTEGER PRIMARY KEY ASC AUTOINCREMENT, ColA INTEGER , ColB TEXT ) ;";
                            cmd.ExecuteNonQuery();


                            cmd.CommandText = "INSERT INTO db.TEST_TABLE VALUES( NULL, 1, 'db1 Record 1') ;";
                            cmd.ExecuteNonQuery();
                            cmd.CommandText = "INSERT INTO db.TEST_TABLE VALUES( NULL, 2, 'db1 Record 2') ;";
                            cmd.ExecuteNonQuery();

                            cmd.CommandText = "INSERT INTO db2.TEST_TABLE VALUES( NULL, 1, 'db2 Record 1') ;";
                            cmd.ExecuteNonQuery();
                            cmd.CommandText = "INSERT INTO db2.TEST_TABLE VALUES( NULL, 2, 'db2 Record 2') ;";
                            cmd.ExecuteNonQuery();

                            cmd.CommandText = "SELECT a.ColB || ' , ' || b.ColB AS [Concatenated] FROM db.TEST_TABLE a JOIN db2.TEST_TABLE b ON a.ColA = b.ColA LIMIT 1 OFFSET 1 ;";
                            SqlDatabaseDataReader dr = cmd.ExecuteReader();
                            dr.Read();
                            lblOutput.Text = dr.GetString(0);

                            dr.Close();

                            cmd.Dispose(); //Optional
                        }

                    }
                    catch (System.Exception ex)
                    {
                        System.Diagnostics.Debug.WriteLine(ex.Message);
                    }
                    finally
                    {

                        if (cnn.State == ConnectionState.Open)
                            cnn.Close();

                    }
                }

            }

            //use memory stream dictionary as you like e.g. saving to disk or network
            // close it
            dictionaryMemoryStream.Clear();


        }

        private void DictionaryStream_Click(object sender, System.EventArgs e)
        {
            Dictionary<string, Stream> dictionaryStream = new Dictionary<string, Stream>();

            string fileName1 = Path.Combine(StoragePath, "dataFile1.db"); //Primary database

            FileStream fs1 = new FileStream(fileName1, FileMode.OpenOrCreate);
            dictionaryStream.Add(fileName1, fs1);

            string fileName2 = Path.Combine(StoragePath, "dataFile2.db");
            FileStream fs2 = new FileStream(fileName2, FileMode.OpenOrCreate);
            dictionaryStream.Add(fileName2, fs2);

            using (SqlDatabaseConnection cnn = new SqlDatabaseConnection())
            {

                cnn.DictionaryStringStream = dictionaryStream;

                cnn.ConnectionString = "SchemaName=db;Datasource=file://" + fileName1 + ";"; //Provide primary file name

                try
                {
                    cnn.Open();
                }
                catch (System.Exception ex)
                {
                    System.Diagnostics.Debug.WriteLine(ex.Message);
                }

                if (cnn.State == ConnectionState.Open)
                {
                    try
                    {
                        using (SqlDatabaseCommand cmd = new SqlDatabaseCommand())
                        {
                            cmd.Connection = cnn;

                            //cmd.CommandText = "DROP TABLE IF EXISTS TEST_TABLE;";
                            //cmd.ExecuteNonQuery();

                            cmd.CommandText = "CREATE TABLE IF NOT EXISTS  db.TEST_TABLE (AutoIncId INTEGER PRIMARY KEY ASC AUTOINCREMENT, ColA INTEGER , ColB TEXT ) ;";
                            cmd.ExecuteNonQuery();

                            //Attach the second file
                            cmd.CommandText = "ATTACH DATABASE '" + fileName2 + "' AS 'db2' ; "; //db2 is the SchemaName and first is the file name to attach.
                            cmd.ExecuteNonQuery();

                            // use SchemaName . Object
                            cmd.CommandText = "CREATE TABLE IF NOT EXISTS db2.TEST_TABLE (AutoIncId INTEGER PRIMARY KEY ASC AUTOINCREMENT, ColA INTEGER , ColB TEXT ) ;";
                            cmd.ExecuteNonQuery();


                            cmd.CommandText = "INSERT INTO db.TEST_TABLE VALUES( NULL, 1, 'db1 Record 1') ;";
                            cmd.ExecuteNonQuery();
                            cmd.CommandText = "INSERT INTO db.TEST_TABLE VALUES( NULL, 2, 'db1 Record 2') ;";
                            cmd.ExecuteNonQuery();

                            cmd.CommandText = "INSERT INTO db2.TEST_TABLE VALUES( NULL, 1, 'db2 Record 1') ;";
                            cmd.ExecuteNonQuery();
                            cmd.CommandText = "INSERT INTO db2.TEST_TABLE VALUES( NULL, 2, 'db2 Record 2') ;";
                            cmd.ExecuteNonQuery();

                            cmd.CommandText = "SELECT a.ColB || ' , ' || b.ColB AS [Concatenated] FROM db.TEST_TABLE a JOIN db2.TEST_TABLE b ON a.ColA = b.ColA LIMIT 1 OFFSET 1 ;";
                            SqlDatabaseDataReader dr = cmd.ExecuteReader();
                            dr.Read();
                            lblOutput.Text = dr.GetString(0);

                            dr.Close();

                            cmd.Dispose(); //Optional
                        }

                    }
                    catch (System.Exception ex)
                    {
                        System.Diagnostics.Debug.WriteLine(ex.Message);
                    }
                    finally
                    {

                        if (cnn.State == ConnectionState.Open)
                            cnn.Close();

                    }
                }

            }
            fs1.Close();
            fs1.Close();
            dictionaryStream.Clear();

        }

        private void StreamObject_Click(object sender, System.EventArgs e)
        {
            string fileName = Path.Combine(StoragePath, "data.db");

            FileStream fs = new FileStream(fileName, FileMode.OpenOrCreate);

            using (SqlDatabaseConnection cnn = new SqlDatabaseConnection())
            {
                cnn.ConnectionString = "SchemaName=db;Datasource=" + fileName + ";";
                cnn.Stream = fs;
                try
                {
                    cnn.Open();
                }
                catch (System.Exception ex)
                {
                    System.Diagnostics.Debug.WriteLine(ex.Message);
                }

                if (cnn.State == ConnectionState.Open)
                {
                    try
                    {
                        using (SqlDatabaseCommand cmd = new SqlDatabaseCommand())
                        {
                            cmd.Connection = cnn;

                            //cmd.CommandText = "DROP TABLE IF EXISTS TEST_TABLE;";
                            //cmd.ExecuteNonQuery();

                            cmd.CommandText = "CREATE TABLE IF NOT EXISTS TEST_TABLE (AutoIncId INTEGER PRIMARY KEY ASC AUTOINCREMENT, ColA INTEGER , ColB TEXT ) ;";
                            cmd.ExecuteNonQuery();

                            int ColACount = 0; //Function and Execute Scalar 
                            cmd.CommandText = "SELECT IfNull(COLA, 0) AS ColA FROM TEST_TABLE ORDER BY COLA DESC LIMIT 1 ;";
                            object obj = cmd.ExecuteScalar();

                            if (obj != null)
                                int.TryParse(obj.ToString(), out ColACount);

                            ColACount += 10; //increment 10 
                            cmd.CommandText = "INSERT INTO db.TEST_TABLE VALUES( NULL, " + ColACount + " , 'SQLDatabase.Net Record 1') ;";
                            cmd.ExecuteNonQuery();

                            ColACount += 10;
                            cmd.CommandText = "INSERT INTO db.TEST_TABLE VALUES( NULL, " + ColACount + " , @data) ;";
                            cmd.Parameters.Add(new SqlDatabaseParameter { ParameterName = "@data", DbType = DbType.Text, Value = "Record 2" });
                            cmd.ExecuteNonQuery();

                            cmd.CommandText = "SELECT AutoIncId, ColA, ColB FROM TEST_TABLE;";
                            SqlDatabaseDataReader dr = cmd.ExecuteReader();

                            int RecordCount = 0;
                            while (dr.Read())
                            {
                                RecordCount++;
                                // VALUES can be accessed by name of index

                                //dr["AutoIncId"] ;                         
                                //dr["ColA"] ;
                                //dr["ColB"].ToString();
                                System.Diagnostics.Debug.WriteLine(dr[0]);
                                System.Diagnostics.Debug.WriteLine(dr[1]);
                                System.Diagnostics.Debug.WriteLine(dr[2]);

                            }
                            dr.Close();

                            cmd.Dispose(); //Optional
                        }

                    }
                    catch (System.Exception ex)
                    {
                        System.Diagnostics.Debug.WriteLine(ex.Message);
                    }
                    finally
                    {

                        if (cnn.State == ConnectionState.Open)
                            cnn.Close();

                        //Another way of closing stream along with connection
                        //lock (cnn.StreamLockObject)
                        //{
                        //    if (fs != null)
                        //        fs.Close();
                        //}
                    }
                }

            }
            if (fs != null)
                fs.Close();

            lblOutput.Text = "Stream Object Completed.";
        }

        private void AtMemory_Click(object sender, System.EventArgs e)
        {

            using (SqlDatabaseConnection cnn = new SqlDatabaseConnection())
            {
                cnn.ConnectionString = "SchemaName=db;Datasource=@memory;";

                try
                {
                    cnn.Open();
                }
                catch (System.Exception ex)
                {
                    System.Diagnostics.Debug.WriteLine(ex.Message);
                }

                if (cnn.State == ConnectionState.Open)
                {
                    try
                    {
                        using (SqlDatabaseCommand cmd = new SqlDatabaseCommand())
                        {
                            cmd.Connection = cnn;
                            cmd.CommandText = "DROP TABLE IF EXISTS TEST_TABLE;";
                            cmd.ExecuteNonQuery();
                            cmd.CommandText = "CREATE TABLE IF NOT EXISTS TEST_TABLE ( ColA INTEGER , ColB TEXT ) ;";
                            cmd.ExecuteNonQuery();

                            cmd.CommandText = "INSERT INTO db.TEST_TABLE VALUES(1, 'SQLDatabase.Net Record 1') ;";
                            cmd.ExecuteNonQuery();

                            cmd.CommandText = "INSERT INTO db.TEST_TABLE VALUES(2, @data) ;";
                            cmd.Parameters.Add(new SqlDatabaseParameter { ParameterName = "@data", DbType = DbType.Text, Value = "Record 2" });
                            cmd.ExecuteNonQuery();

                            cmd.CommandText = "SELECT ColA, ColB FROM TEST_TABLE;";
                            SqlDatabaseDataReader dr = cmd.ExecuteReader();

                            int RecordCount = 0;
                            while (dr.Read())
                            {
                                //dr["ColA"] ;
                                //dr["ColB"].ToString();
                                RecordCount++;
                            }
                            dr.Close();
                                                        
                            cmd.Dispose(); //Optional
                        }

                    }
                    catch (System.Exception ex)
                    {
                        System.Diagnostics.Debug.WriteLine(ex.Message);
                    }
                    finally
                    {
                        if (cnn.State == ConnectionState.Open)
                            cnn.Close();
                    }
                }

            }
            lblOutput.Text = "At Memory Completed.";
        }

    }
}

